/**
 *******************************************************************************
 * @file    tmpm364_can_int.c
 * @brief   All interrupt request functions definition of CAN
 *          for the TOSHIBA 'TMPM364' Device Series 
 * @version V0.101
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm364_can_int.h"

/**
  * @brief  CAN mailbox Rx interrupt.
  * @param  None
  * @retval None
  */
void INTCANRX_IRQHandler(void)
{
    CAN_MbxState mbxstate = { 0U };
    CAN_MsgTypeDef msgrx = { 0U };

    mbxstate = CAN_GetMbxTRxState(CAN_STATE_ID_RX_ACK); /* Read Rx state */
    if (mbxstate.Bit.Mbx10 == 1U) {     /* If mailbox10 has received a message */
        CAN_GetMbxMsg(CAN_MBX_10, &msgrx);     /* Read mailbox10 */
        LED_Display(msgrx.MsgData[0] >> 4U);   /* Get the data[0], and displays on LED PM4~7 */
    } else {
        /* Do nothing */
    }
    CAN_ClearMbxTRxState(CAN_STATE_ID_RX_ACK);  /* Clear the Rx state */
    CAN_ClearINTFlag(CAN_INT_TYPE_RX);  /* Clear the Rx interrupt flag */
}
