/**
 *******************************************************************************
 * @file    main.c
 * @brief   The demo functions of CG for the TOSHIBA 'TMPM364' Device Series 
 * @version V0.201
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "tmpm364_cg.h"

#define C200US  0x0096U         /* @12M Oscillator */

Result CG_EnableClkMulCircuit(void);

int main(void)
{
    /*SystemInit(); */
    /* Set CG Module  */
    /* set fgear = fc/2 */
    CG_SetFgearLevel(CG_DIVIDE_2);
    /* set fperi = fgear  fpreclk = fperi/32 */
#if defined (__TMPM361_CG_H)||defined(__TMPM362_CG_H)||defined(__TMPM363_CG_H)||defined(__TMPM364_CG_H)
    CG_SetPhiT0Src(CG_PHIT0_SRC_FPSCALER);
#endif
    CG_SetPhiT0Level(CG_DIVIDE_64);
    /* set SCOUT source to fT0 */
    CG_SetSCOUTSrc(CG_SCOUT_SRC_PHIT0);
    /* enable high-speed oscillation */
    CG_SetFosc(ENABLE);
    /* enable low-speed oscillation */
    CG_SetFs(ENABLE);
    /* set low power consumption mode sleep */
    CG_SetSTBYMode(CG_STBY_MODE_SLEEP);
    /* set high-speed and low-speed oscillation to be enabled after releasing stop mode */
    CG_SetExitStopModeFosc(ENABLE);
    CG_SetExitStopModeFs(ENABLE);
    /* set pin status in stop mode to "active" */
    CG_SetPinStateInStopMode(ENABLE);
    /*set up pll and wait 200us for pll to warm up , set fc source to fpll */
    CG_EnableClkMulCircuit();
    /* set system clock to fgear */
    CG_SetFsysSrc(CG_FSYS_SRC_FGEAR);

    return 0;
}

Result CG_EnableClkMulCircuit(void)
{

    Result retval = ERROR;
    WorkState st = BUSY;

    CG_SetClkMulTimes(CG_MUL_4TIMES);
    retval = CG_SetPLL(ENABLE);

    if (retval == SUCCESS) {
        /*set warm up time to about 200us */
        CG_SetWarmUpTime(CG_WARM_UP_SRC_X1, C200US);
        CG_StartWarmUp();
        /*wait warm up to end */
        do {
            st = CG_GetWarmUpState();
        } while (st != DONE);
        retval = CG_SetFcSrc(CG_FC_SRC_FPLL);

    } else {
        /*Do nothing */
    }
    return retval;
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
